function [Data,FN]=Lab0GetData(filename)
% This program will extract data and return several useful anonymous 
% function handles.

if filename==0, %no filename was specified 
    filename = getfilename();
elseif ~exist('filename','file')
    warning('I cannot find the file you requested in this directory')
    filename = getfilename();
end

%Get the raw data
Raw_Data = extract_data(filename); % from the file lab4_raw_impulse_data
      
%convert to centimeters
E2M   = 1/2266; %Instructors manual page 56. encoder to cm conversion factor 

Data.T    =     Raw_Data(:, 2);
Data.C    =     Raw_Data(:, 3);
Data.E{1} = E2M*Raw_Data(:, 4);
Data.E{2} = E2M*Raw_Data(:, 5);
Data.E{3} = E2M*Raw_Data(:, 6);

%create a useful set of anonymous functions
FN.Readme = sprintf(['\n\nAll of these functions from from Lab0GetData.m \n'...
         'Contents:  [XX,YY]    =FN.Interp(X,Y) - Resolved data; splined\n'...
         '           [Xmax,Ymax]=FN.Peak(X,Y)   - Estimate peak location (for fft) \n'...
         '           [Amp,Freq] =FN.FFT(T,Y)    - Fast Fourier Transform (Freq in Hz)\n'...
         '           [Vel,Acc]  =FN.d_dt(T,Y)   - Find Velocity and Acceleration \n\n']);
FN.Interp = @(X,Y) spline_interpolation(X,Y);
FN.Peak   = @(X,Y) spline_peak_detector(X,Y);
FN.FFT    = @(T,Y) FFT(T,Y);
FN.d_dt   = @(T,Y) Derivatives(T,Y);

%Setup the figure
close all
figure1 = figure('InvertHardcopy','off',...
                    'Color',[1 1 1]);
axes1 = axes('Parent',figure1,'YGrid','on','XGrid','on',...
               ...% 'Units','centimeters',...
                'MinorGridLineStyle','--',...
                'GridLineStyle','-',...
                'FontSize',8,...
                'FontName','times new roman');
box(axes1,'on');
hold(axes1,'all');

end

function [XX,YY]= spline_interpolation(X,Y)
    XX=[X(1):0.001:X(end)];
    YY=spline(X,Y,XX);  
end 

function [Xmax,Ymax]= spline_peak_detector(X,Y)
    Y(1:3)=0; %delete any DC offset
    
    %Estimate peak location
    [~,maxI] = max(Y);
    if maxI < 3, Xmax=0; Ymax=0; return, end
    XX = X([maxI-2,maxI+2]);
    
    % spline based interpolation of the FFT
    XX=XX(1):0.01:XX(2);
    YY=spline(X,Y,XX);
    
    %Find max
    [Ymax,Imax]=max(YY);
     Xmax      =XX(Imax);
    
end 

function [Amp,Frequency]=FFT(T,Y)
    TT = T(5)-T(4);                     % Sample time
    Fs = 1/TT;
    L = length(Y);
    NFFT = 2^nextpow2(L);               % Not necessary in 2015!
    
    
    Frequency = Fs/2*linspace(0,1,NFFT/2+1);   %only up to Nyquist
    Amp = fft(Y.*hann(L),NFFT)/L;
    Amp = Amp(1:length(Frequency));
    
    %remove data above the Nyquist frequency
end

function [V,A]=Derivatives(T,Y)
%Calculate velocity and acceleration for Y
       V = 1/2*[0;diff(Y(1:end-1))./diff(T(1:end-1))+...
                    diff(Y(2:end))./diff(T(2:end));0];
       A = [0;diff(diff(Y));0];
end

function Raw_Data = extract_data(filename)
% The file lab4_raw_impulse_data needs the following modification
% line 1 needs to be deleted
% line 2 needs to be deleted
% line 3 needs to have "Raw_Data=" prepended

%Open & read the filethe file
  fid = fopen(filename, 'r');
    c = fread(fid,  'uint8=>char')';
        fclose(fid);  
%Edit the loaded file (replace everything until '[' with 'Raw_Data='
    c = regexprep(c,'[^\[]+','Raw_Data = ','once');
    
%Silence the output
    c = regexprep(c,'\]','];');
%Execute the variable definition 
   eval(c); %This creates the variable Raw_Data
   
% This file was modified by Geoff Recktenwald 7-23-2015
end    
   
function [filename]=getfilename()
% Get a list of filenames in the current directory
    directory_files = dir;
    
    %Ignore matlab scripts based on size
    ii=0;
    for i=1:1:length(directory_files)
       if directory_files(i).bytes>5000, %too small for data
         ii=ii+1;
           filenames{ii}=directory_files(i).name;
       end
    end
    
    filenames{end+1}='File not displayed';
    [index,v] = listdlg('PromptString','Select a file:',...
                    'SelectionMode','single',...
                    'ListString',filenames);
    if index==length(filenames)
        fprintf(['\n If your filename is not here please save your data'...
                 '\n and Matlab files in the same directory.  Then navigate'...
                 '\n to that directory before running this program.\n\n']);
        error('You are in the wrong directory')
    end
        
    filename=filenames{index};            
end
 